@@ -3,12 +3,15 @@ use warnings;
 
 package JMAP::Tester;
 # ABSTRACT: a JMAP client made for testing JMAP servers
-$JMAP::Tester::VERSION = '0.001';
+$JMAP::Tester::VERSION = '0.002';
 use Moo;
 
 use Encode qw(encode_utf8);
+use HTTP::Request;
 use JMAP::Tester::Response;
 use JMAP::Tester::Result::Failure;
+use Safe::Isa;
+use URI;
 
 #pod =head1 OVERVIEW
 #pod
@@ -53,6 +56,12 @@ use JMAP::Tester::Result::Failure;
 #pod   my $struct = $response->as_struct;   # gets the complete JSON data
 #pod   $jtest->strip_json_types( $struct ); # strips all the JSON::Typist types
 #pod
+#pod Or more simply:
+#pod
+#pod   my $struct = $response->as_stripped_struct;
+#pod
+#pod There is also L<JMAP::Tester::Response/"as_stripped_pairs">.
+#pod
 #pod =cut
 
 has json_codec => (
@@ -67,8 +76,8 @@ has json_codec => (
   },
 );
 
-has json_typist => (
-  is => 'bare',
+has _json_typist => (
+  is => 'ro',
   handles => {
     apply_json_types => 'apply_types',
     strip_json_types => 'strip_types',
@@ -81,23 +90,49 @@ has json_typist => (
 
 has jmap_uri => (
   is => 'ro',
+  isa => sub {
+    die "provided jmap_uri is not a URI object" unless $_[0]->$_isa('URI');
+  },
+  coerce => sub {
+    return $_[0] if $_[0]->$_isa('URI');
+    die "can't use reference as a URI" if ref $_[0];
+    URI->new($_[0]);
+  },
   required => 1,
 );
 
-has _request_callback => (
-  is => 'ro',
+has ua => (
+  is   => 'ro',
+  lazy => 1,
   default => sub {
     require LWP::UserAgent;
     my $ua = LWP::UserAgent->new;
+    $ua->cookie_jar({});
 
     if ($ENV{IGNORE_INVALID_CERT}) {
       $ua->ssl_opts(SSL_verify_mode => 0, verify_hostname => 0);
     }
 
-    return sub { my $self = shift; $ua->request(@_) }
+    return $ua;
   },
 );
 
+sub _set_cookie {
+  my ($self, $name, $value) = @_;
+  $self->ua->cookie_jar->set_cookie(
+    1,
+    $name,
+    $value,
+    $self->jmap_uri->path,
+    $self->jmap_uri->host,
+    $self->jmap_uri->port,
+    0,
+    0,
+    86400,
+    0,
+  );
+}
+
 #pod =method request
 #pod
 #pod   my $result = $jtest->request([
@@ -107,15 +142,15 @@ has _request_callback => (
 #pod
 #pod This method takes an arrayref of method calls and sends them to the JMAP
 #pod server.  If the method calls have a third element (a I<client id>) then it's
-#pod left as-is.  If no client id is given, one is generated.  You can mixing explicit
-#pod and autogenerated client ids.  They will never conflict.
+#pod left as-is.  If no client id is given, one is generated.  You can mix
+#pod explicit and autogenerated client ids.  They will never conflict.
 #pod
-#pod The arguments to methods are JSON-encoding with a L<JSON::Typist>-aware
+#pod The arguments to methods are JSON-encoded with a L<JSON::Typist>-aware
 #pod encoder, so JSON::Typist types can be used to ensure string or number types in
 #pod the generated JSON.
 #pod
 #pod The return value is an object that does the L<JMAP::Tester::Result> role,
-#pod meaning it's got an C<is_success> method that return true or false.  For now,
+#pod meaning it's got an C<is_success> method that returns true or false.  For now,
 #pod at least, failures are L<JMAP::Tester::Result::Failure> objects.  More refined
 #pod failure objects may exist in the future.  Successful requests return
 #pod L<JMAP::Tester::Response> objects.
@@ -145,15 +180,14 @@ sub request {
   my $json = $self->json_encode(\@suffixed);
 
   my $post = HTTP::Request->new(
-    POST => $self->jmap_uri,
+    POST => $self->jmap_uri->as_string,
     [
       'Content-Type' => 'application/json',
     ],
     encode_utf8($json),
   );
 
-  my $request_cb = $self->_request_callback;
-  my $http_res = $self->$request_cb($post);
+  my $http_res = $self->ua->request($post);
 
   unless ($http_res->is_success) {
     return JMAP::Tester::Result::Failure->new({
@@ -169,6 +203,7 @@ sub request {
 
   return JMAP::Tester::Response->new({
     struct => $data,
+    _json_typist => $self->_json_typist,
   });
 }
 
@@ -186,7 +221,7 @@ JMAP::Tester - a JMAP client made for testing JMAP servers
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 OVERVIEW
 
@@ -231,6 +266,12 @@ Perl code on them.  You can do that with:
   my $struct = $response->as_struct;   # gets the complete JSON data
   $jtest->strip_json_types( $struct ); # strips all the JSON::Typist types
 
+Or more simply:
+
+  my $struct = $response->as_stripped_struct;
+
+There is also L<JMAP::Tester::Response/"as_stripped_pairs">.
+
 =head1 METHODS
 
 =head2 request
@@ -242,15 +283,15 @@ Perl code on them.  You can do that with:
 
 This method takes an arrayref of method calls and sends them to the JMAP
 server.  If the method calls have a third element (a I<client id>) then it's
-left as-is.  If no client id is given, one is generated.  You can mixing explicit
-and autogenerated client ids.  They will never conflict.
+left as-is.  If no client id is given, one is generated.  You can mix
+explicit and autogenerated client ids.  They will never conflict.
 
-The arguments to methods are JSON-encoding with a L<JSON::Typist>-aware
+The arguments to methods are JSON-encoded with a L<JSON::Typist>-aware
 encoder, so JSON::Typist types can be used to ensure string or number types in
 the generated JSON.
 
 The return value is an object that does the L<JMAP::Tester::Result> role,
-meaning it's got an C<is_success> method that return true or false.  For now,
+meaning it's got an C<is_success> method that returns true or false.  For now,
 at least, failures are L<JMAP::Tester::Result::Failure> objects.  More refined
 failure objects may exist in the future.  Successful requests return
 L<JMAP::Tester::Response> objects.
@@ -259,12 +300,22 @@ L<JMAP::Tester::Response> objects.
 
 Ricardo SIGNES <rjbs@cpan.org>
 
-=head1 CONTRIBUTOR
+=head1 CONTRIBUTORS
 
-=for stopwords Alfie John
+=for stopwords Alfie John Matthew Horsfall
+
+=over 4
+
+=item *
 
 Alfie John <alfiej@fastmail.fm>
 
+=item *
+
+Matthew Horsfall <wolfsage@gmail.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2016 by FastMail, Ltd.
