@@ -2,7 +2,7 @@ use v5.10.0;
 
 package JMAP::Tester::Response;
 # ABSTRACT: what you get in reply to a succesful JMAP request
-$JMAP::Tester::Response::VERSION = '0.001';
+$JMAP::Tester::Response::VERSION = '0.002';
 use Moo;
 with 'JMAP::Tester::Role::Result';
 
@@ -27,6 +27,13 @@ has struct => (
   required => 1,
 );
 
+has _json_typist => (
+  is => 'ro',
+  handles => {
+    _strip_json_types => 'strip_types',
+  },
+);
+
 sub BUILD {
   $_[0]->_index_setup;
 }
@@ -88,29 +95,42 @@ has para_indices => (is => 'bare', accessor => '_para_indices');
 sub sentence {
   my ($self, $n) = @_;
   return unless my $triple = $self->_struct->[$n];
-  return JMAP::Tester::Response::Sentence->new($triple);
+  return JMAP::Tester::Response::Sentence->new({
+    triple => $triple,
+    _json_typist => $self->_json_typist,
+  });
 }
 
 #pod =method single_sentence
 #pod
 #pod   my $sentence = $response->single_sentence;
+#pod   my $sentence = $response->single_sentence($name);
 #pod
 #pod This method returns the only L<Sentence|JMAP::Tester::Response::Sentence> of
-#pod the response, raising an exception of there's more than one Sentence.
+#pod the response, raising an exception if there's more than one Sentence.  If
+#pod C<$name> is given, an exception is raised if the Sentence's name doesn't match
+#pod the given name.
 #pod
 #pod =cut
 
 sub single_sentence {
-  my ($self) = @_;
+  my ($self, $name) = @_;
 
-  my @triples = @{ $self->_struct };
-  unless (@triples == 1) {
+  my @sentences = @{ $self->_struct };
+  unless (@sentences == 1) {
     Carp::confess(
-      sprint("single_sentence called but there are %i sentences", 0+@triples)
+      sprintf("single_sentence called but there are %i sentences", 0+@sentences)
     );
   }
 
-  return JMAP::Tester::Response::Sentence->new($triples[0]);
+  if (defined $name && $sentences[0][0] ne $name) {
+    Carp::confess(qq{single sentence has name "$sentences[0][0]" not "$name"});
+  }
+
+  return JMAP::Tester::Response::Sentence->new({
+    triple       => $sentences[0],
+    _json_typist => $self->_json_typist
+  });
 }
 
 #pod =method paragraph
@@ -128,7 +148,12 @@ sub paragraph {
   return unless my $indices = $self->_para_indices->[$n];
   my @triples = @{ $self->_struct }[ @$indices ];
   return JMAP::Tester::Response::Paragraph->new({
-    sentences => [ map {; JMAP::Tester::Response::Sentence->new($_) } @triples ],
+    sentences => [ map {;
+      JMAP::Tester::Response::Sentence->new({
+        triple => $_,
+        _json_typist => $self->_json_typist
+      }) } @triples ],
+    _json_typist => $self->_json_typist,
   });
 }
 
@@ -155,8 +180,13 @@ sub assert_n_paragraphs {
   for my $i_set (@para_indices) {
     push @sets, JMAP::Tester::Response::Paragraph->new({
       sentences => [
-        map {; JMAP::Tester::Response::Sentence->new($_) } @{$res}[ @$i_set ]
+        map {;
+          JMAP::Tester::Response::Sentence->new({ 
+            triple => $_,
+            _json_typist => $self->_json_typist
+           }) } @{$res}[ @$i_set ]
       ],
+      _json_typist => $self->_json_typist,
     });
   }
 
@@ -178,15 +208,22 @@ sub paragraph_by_client_id {
   return unless my $indices = $self->_cid_indices->{$cid};
   my @triples = @{ $self->_struct }[ @$indices ];
   return JMAP::Tester::Response::Paragraph->new({
-    sentences => [ map {; JMAP::Tester::Response::Sentence->new($_) } @triples ],
+    sentences => [ map {;
+      JMAP::Tester::Response::Sentence->new({
+        triple => $_,
+        _json_typist => $self->_json_typist
+      }) } @triples ],
+    _json_typist => $self->_json_typist,
   });
 }
 
 #pod =method as_struct
 #pod
+#pod =method as_stripped_struct
+#pod
 #pod This method returns an arrayref of arrayrefs, holding the data returned by the
-#pod JMAP server.  Some of the JSON data may be in objects provided by
-#pod L<JSON::Typist>, so consider stripping types!
+#pod JMAP server.  With C<as_struct>, some of the JSON data may be in objects provided by
+#pod L<JSON::Typist>. If you'd prefer raw data, use the C<as_stripped_struct> form.
 #pod
 #pod =cut
 
@@ -194,14 +231,23 @@ sub as_struct {
   my ($self) = @_;
 
   return [
-    map {; JMAP::Tester::Response::Sentence->new($_)->as_struct }
+    map {; JMAP::Tester::Response::Sentence->new({triple => $_})->as_struct }
     @{ $self->_struct }
   ];
 }
 
+sub as_stripped_struct {
+  my ($self) = @_;
+
+  return $self->_strip_json_types($self->as_struct);
+}
+
 #pod =method as_pairs
 #pod
-#pod This method does the same thing as C<as_struct>, but omits client ids.
+#pod =method as_stripped_pairs
+#pod
+#pod These methods do the same thing as C<as_struct> and <as_stripped_struct>,
+#pod but omit client ids.
 #pod
 #pod =cut
 
@@ -209,11 +255,17 @@ sub as_pairs {
   my ($self) = @_;
 
   return [
-    map {; JMAP::Tester::Response::Sentence->new($_)->as_pair }
+    map {; JMAP::Tester::Response::Sentence->new({triple => $_})->as_pair }
     @{ $self->_struct }
   ];
 }
 
+sub as_stripped_pairs {
+  my ($self) = @_;
+
+  return $self->_strip_json_types($self->as_pairs);
+}
+
 1;
 
 __END__
@@ -228,7 +280,7 @@ JMAP::Tester::Response - what you get in reply to a succesful JMAP request
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 OVERVIEW
 
@@ -250,9 +302,12 @@ the response.
 =head2 single_sentence
 
   my $sentence = $response->single_sentence;
+  my $sentence = $response->single_sentence($name);
 
 This method returns the only L<Sentence|JMAP::Tester::Response::Sentence> of
-the response, raising an exception of there's more than one Sentence.
+the response, raising an exception if there's more than one Sentence.  If
+C<$name> is given, an exception is raised if the Sentence's name doesn't match
+the given name.
 
 =head2 paragraph
 
@@ -277,13 +332,18 @@ for that client id, an empty list is returned.
 
 =head2 as_struct
 
+=head2 as_stripped_struct
+
 This method returns an arrayref of arrayrefs, holding the data returned by the
-JMAP server.  Some of the JSON data may be in objects provided by
-L<JSON::Typist>, so consider stripping types!
+JMAP server.  With C<as_struct>, some of the JSON data may be in objects provided by
+L<JSON::Typist>. If you'd prefer raw data, use the C<as_stripped_struct> form.
 
 =head2 as_pairs
 
-This method does the same thing as C<as_struct>, but omits client ids.
+=head2 as_stripped_pairs
+
+These methods do the same thing as C<as_struct> and <as_stripped_struct>,
+but omit client ids.
 
 =head1 AUTHOR
 
