@@ -1,7 +1,7 @@
 use v5.10.0;
 package JMAP::Tester::Response::Paragraph;
 # ABSTRACT: a group of sentences in a JMAP response
-$JMAP::Tester::Response::Paragraph::VERSION = '0.001';
+$JMAP::Tester::Response::Paragraph::VERSION = '0.002';
 use Moo;
 
 #pod =head1 OVERVIEW
@@ -22,6 +22,13 @@ use Moo;
 
 has sentences => (is => 'bare', reader => '_sentences', required => 1);
 
+has _json_typist => (
+  is => 'ro',
+  handles => {
+    _strip_json_types => 'strip_types',
+  },
+);
+
 #pod =method sentences
 #pod
 #pod The C<sentences> method returns a list of
@@ -32,6 +39,19 @@ has sentences => (is => 'bare', reader => '_sentences', required => 1);
 
 sub sentences { @{ $_[0]->_sentences } }
 
+#pod =method sentence
+#pod
+#pod   my $sentence = $para->sentence($n);
+#pod
+#pod This method returns the I<n>th sentence of the paragraph.
+#pod
+#pod =cut
+
+sub sentence {
+  # die on out-of-range?
+  $_[0]->_sentences->[$_[1]];
+}
+
 #pod =method single
 #pod
 #pod   my $sentence = $para->single;
@@ -41,7 +61,7 @@ sub sentences { @{ $_[0]->_sentences } }
 #pod paragraph.  If a C<$name> argument is given and the paragraph's single
 #pod sentence doesn't have that name, an exception is raised.
 #pod
-#pod Otherwise, this method reeturns the sentence.
+#pod Otherwise, this method returns the sentence.
 #pod
 #pod =cut
 
@@ -61,8 +81,10 @@ sub single {
 
 #pod =method as_struct
 #pod
-#pod This method returns an arrayref containing the result of calling C<as_struct>
-#pod on each sentence in the paragraph.
+#pod =method as_stripped_struct
+#pod
+#pod C<as_struct> returns an arrayref containing the result of calling C<as_struct>
+#pod on each sentence in the paragraph. C<as_stripped_struct> removes JSON types.
 #pod
 #pod =cut
 
@@ -70,10 +92,14 @@ sub as_struct {
   [ map {; $_->as_struct } $_[0]->sentences ]
 }
 
+sub as_stripped_struct {
+  $_[0]->_strip_json_types($_[0]->as_struct);
+}
+
 #pod =method as_pairs
 #pod
-#pod This method returns an arrayref containing the result of calling C<as_pair>
-#pod on each sentence in the paragraph.
+#pod C<as_pairs> returns an arrayref containing the result of calling C<as_pair>
+#pod on each sentence in the paragraph. C<as_stripped_pairs> removes JSON types.
 #pod
 #pod =cut
 
@@ -81,6 +107,10 @@ sub as_pairs {
   [ map {; $_->as_pair } $_[0]->sentences ]
 }
 
+sub as_stripped_pairs {
+  $_[0]->_strip_json_types($_[0]->as_pairs);
+}
+
 1;
 
 __END__
@@ -95,7 +125,7 @@ JMAP::Tester::Response::Paragraph - a group of sentences in a JMAP response
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 OVERVIEW
 
@@ -119,6 +149,12 @@ The C<sentences> method returns a list of
 L<Sentence|JMAP::Tester::Response::Sentence> objects, one for each sentence
 in the paragraph.
 
+=head2 sentence
+
+  my $sentence = $para->sentence($n);
+
+This method returns the I<n>th sentence of the paragraph.
+
 =head2 single
 
   my $sentence = $para->single;
@@ -128,17 +164,19 @@ This method throws an exception if there is more than one sentence in the
 paragraph.  If a C<$name> argument is given and the paragraph's single
 sentence doesn't have that name, an exception is raised.
 
-Otherwise, this method reeturns the sentence.
+Otherwise, this method returns the sentence.
 
 =head2 as_struct
 
-This method returns an arrayref containing the result of calling C<as_struct>
-on each sentence in the paragraph.
+=head2 as_stripped_struct
+
+C<as_struct> returns an arrayref containing the result of calling C<as_struct>
+on each sentence in the paragraph. C<as_stripped_struct> removes JSON types.
 
 =head2 as_pairs
 
-This method returns an arrayref containing the result of calling C<as_pair>
-on each sentence in the paragraph.
+C<as_pairs> returns an arrayref containing the result of calling C<as_pair>
+on each sentence in the paragraph. C<as_stripped_pairs> removes JSON types.
 
 =head1 AUTHOR
 
