@@ -1,7 +1,7 @@
 use v5.10.0;
 package JMAP::Tester::Response::Sentence;
 # ABSTRACT: a single triple within a JMAP response
-$JMAP::Tester::Response::Sentence::VERSION = '0.001';
+$JMAP::Tester::Response::Sentence::VERSION = '0.002';
 use Moo;
 
 #pod =head1 OVERVIEW
@@ -25,11 +25,14 @@ use Moo;
 
 sub BUILDARGS {
   my ($self, $args) = @_;
-  if (ref $args && ref $args eq 'ARRAY') {
+
+  if (my $triple = delete $args->{triple}) {
     return {
-      name => $args->[0],
-      arguments => $args->[1],
-      client_id => $args->[2],
+      %$args,
+
+      name      => $triple->[0],
+      arguments => $triple->[1],
+      client_id => $triple->[2],
     };
   }
   return $self->SUPER::BUILDARGS($args);
@@ -39,26 +42,49 @@ has name      => (is => 'ro', required => 1);
 has arguments => (is => 'ro', required => 1);
 has client_id => (is => 'ro', required => 1);
 
+has _json_typist => (
+  is => 'ro',
+  handles => {
+    _strip_json_types => 'strip_types',
+  },
+);
+
 #pod =method as_struct
 #pod
-#pod This returns the underlying JSON data of the sentence, which may include
+#pod =method as_stripped_struct
+#pod
+#pod C<as_struct> returns the underlying JSON data of the sentence, which may include
 #pod objects used to convey type information for booleans, strings, and numbers.
 #pod
-#pod It returns a three-element arrayref.
+#pod For raw data, use C<as_stripped_struct>.
+#pod
+#pod These return a three-element arrayref.
 #pod
 #pod =cut
 
 sub as_struct { [ $_[0]->name, $_[0]->arguments, $_[0]->client_id ] }
 
+sub as_stripped_struct {
+  $_[0]->_strip_json_types($_[0]->as_struct);
+}
+
 #pod =method as_pair
 #pod
-#pod This method returns the same thing as C<as_struct>, but without the
+#pod =method as_stripped_pair
+#pod
+#pod C<as_pair> returns the same thing as C<as_struct>, but without the
 #pod C<client_id>.  That means it returns a two-element arrayref.
 #pod
+#pod C<as_stripped_pair> returns the same minus JSON type information.
+#pod
 #pod =cut
 
 sub as_pair { [ $_[0]->name, $_[0]->arguments ] }
 
+sub as_stripped_pair {
+  $_[0]->_strip_json_types($_[0]->as_pair);
+}
+
 #pod =method as_set
 #pod
 #pod This method returns a L<JMAP::Tester::Response::Sentence::Set> object for the
@@ -70,9 +96,9 @@ sub as_pair { [ $_[0]->name, $_[0]->arguments ] }
 sub as_set {
   require JMAP::Tester::Response::Sentence::Set;
   return JMAP::Tester::Response::Sentence::Set->new({
-    name      => $_[0]->name,
-    arguments => $_[0]->arguments,
-    client_id => $_[0]->client_id,
+    name         => $_[0]->name,
+    arguments    => $_[0]->arguments,
+    client_id    => $_[0]->client_id,
   });
 }
 
@@ -90,7 +116,7 @@ JMAP::Tester::Response::Sentence - a single triple within a JMAP response
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 OVERVIEW
 
@@ -113,16 +139,24 @@ method.
 
 =head2 as_struct
 
-This returns the underlying JSON data of the sentence, which may include
+=head2 as_stripped_struct
+
+C<as_struct> returns the underlying JSON data of the sentence, which may include
 objects used to convey type information for booleans, strings, and numbers.
 
-It returns a three-element arrayref.
+For raw data, use C<as_stripped_struct>.
+
+These return a three-element arrayref.
 
 =head2 as_pair
 
-This method returns the same thing as C<as_struct>, but without the
+=head2 as_stripped_pair
+
+C<as_pair> returns the same thing as C<as_struct>, but without the
 C<client_id>.  That means it returns a two-element arrayref.
 
+C<as_stripped_pair> returns the same minus JSON type information.
+
 =head2 as_set
 
 This method returns a L<JMAP::Tester::Response::Sentence::Set> object for the
