@@ -1,7 +1,7 @@
 use v5.10.0;
 package JMAP::Tester::Response::Sentence::Set;
 # ABSTRACT: the kind of sentence you get in reply to a setFoos call
-$JMAP::Tester::Response::Sentence::Set::VERSION = '0.001';
+$JMAP::Tester::Response::Sentence::Set::VERSION = '0.002';
 use Moo;
 
 #pod =head1 OVERVIEW
@@ -12,18 +12,6 @@ use Moo;
 #pod
 #pod =cut
 
-sub BUILDARGS {
-  my ($self, $args) = @_;
-  if (ref $args && ref $args eq 'ARRAY') {
-    return {
-      name => $args->[0],
-      arguments => $args->[1],
-      client_id => $args->[2],
-    };
-  }
-  return $self->SUPER::BUILDARGS($args);
-}
-
 has name      => (is => 'ro', required => 1);
 has arguments => (is => 'ro', required => 1);
 has client_id => (is => 'ro', required => 1);
@@ -138,7 +126,7 @@ JMAP::Tester::Response::Sentence::Set - the kind of sentence you get in reply to
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 OVERVIEW
 
